function [AV, tau  ] = AllanVar( time_series, freq  )
%[AV, tau] = AllanVar(time_series, freq) computes the Allan Variance over all possible octaves and makes a
%plot. time_series are the data sampled at freq, e.g. 60 Hz (David
%Jacobson, 12/12/2013)

%Find the number of octaves to use:

octaves = floor(log2(length(time_series)));

tau_c = 1/freq;

%Iterating over each ocatve number

zRaw = getRawZ(time_series);

for oct_num=1:octaves
    %We compute the bin size for this octave number
    m = 2^(oct_num-1);
    
    %We find the value of tau corresponding to this octave:
    tau(oct_num) = m*tau_c;
    
    
    %We now perform the sum eqn. 18:
    %summand = 0;
    %for k=1:(length(time_series)-2*m+1)
    %    summand = summand + (z(k+2*m,time_series,tau(oct_num))-2*z(k+m,time_series,tau(oct_num))+z(k,time_series,tau(oct_num)))^2;
    %end;
    
    z = tau(oct_num)*zRaw;
    
    %summand = 0;
    %for k=1:(length(time_series)-2*m)
    %    summand = summand + (z(k+2*m)-2*z(k+m)+z(k))^2;
    %end;
    
    %The summation can be done most efficiently by using convolutions.  First
    %we define the convolution filter for this particular value of m:
    
    filter = zeros(m*2+1,1);
    filter([1,1+2*m]) = 1;
    filter(1+m) = -2;
    
    
    conv_out = conv(z, filter);
    summands = conv_out((2*m+1):(length(conv_out)-2*m));
    
    
    adduct = sum(summands.^2);
    
    AV(oct_num) = 1/2/(length(time_series)-2*m+1)/(m*tau(oct_num))^2*adduct;
    
end;

%Now plot the AV as a function of tau:

%figure;
%loglog(tau,AV,'.k');

end


function [z] = getRawZ(time_series)
z = zeros(length(time_series),1);
z(1) = 0;
%The cumulative sum function is used to generate the raw z's much faster
%than explicit evaluation of eqn. 18b in Bob's correction:
z(2:length(z)) = cumsum(time_series(1:(length(z)-1)));

end

