function [ k, alpha ] = MLEAVfit( trace, freq )
%[k,alpha] = MLEAVforce(trace,freq) loads a trace at freq, computes the Allan
%variance, fits the function from Lansdorp-2012 and returns the spring
%constant, k, and dissipation, alpha 

%Compute the Allan Variance of the time series:

[AV, tau] = AllanVar(trace, freq);

%Now we will need to do an unconstrained optimization to minimize the MLE
%cost function:

%Compute the trace-length parameters:

N = length(trace);
for k=1:length(tau)
    m(k) = 2^(k-1);
end;
eta = 1/2*(N./m-1);

%We use a least-squres fit to get the approximate parameters used in the
%maximum liklihood calculation:

ffun = fittype('2*4.1*a/(k^2*x)*(1+2*a/(k*x)*exp(-k*x/a)-a/(2*k*x)*exp(-2*k*x/a)-3*a/(2*k*x))');
ffun_limited = fitoptions(ffun);
%Setting fit parameters:
set(ffun_limited, 'Lower', [1E-6,0], 'Upper',[1E-4,Inf], 'StartPoint',[1.5E-5,0.03],'MaxFunEvals',6000,'MaxIter',4000);
ffun = fittype('2*4.1*a/(k^2*x)*(1+2*a/(k*x)*exp(-k*x/a)-a/(2*k*x)*exp(-2*k*x/a)-3*a/(2*k*x))', 'options', ffun_limited);
fitObject = fit(tau.',AV.', ffun);

coefficients = coeffvalues(fitObject);
alpha_guess = coefficients(1);
k_guess = coefficients(2);

%Define the anonymous function for the cost function in terms of two other
%functions defined at the end of this file: x = [a,k]

cost_fun = @(x) cost(x(1),x(2),AV,tau,eta);

[fits,~,flag] = fminsearch(cost_fun,[alpha_guess,k_guess]);

%If the minimum search does not find a min., I return an error.  One could
%also handle this, on bad data where this is a common problem, by using the
%least-squares values obtained above

if flag ~= 1
    error('fminsearch minimization of cost function failed to converge');
end;

k = fits(2);
alpha = fits(1);

%disp(sprintf('Alpha: %g; k: %g',[alpha,k]));

% %Make a plot the check the quality of the fit:
% for n=1:length(tau)
%    AV_fits(n) = AV_ideal(alpha,k,tau(n));
% end;
% 
% 
% figure;
% loglog(tau,AV,'.k');
% hold on;
% loglog(tau,AV_fits,'--r');





end

%Function for the AV from the Langevin equation:

function [AV] = AV_ideal(a,k,t)
kT = 4.1;
AV = 2*kT*a/(k^2*t)*(1+2*a/(k*t)*exp(-k*t/a)-a/(2*k*t)*exp(-2*k*t/a)-3*a/(2*k*t));
end

function [F] = cost(a,k,AV_measured,tau,eta)
for n=1:length(tau)
    term(n) = eta(n)*(AV_measured(n)/AV_ideal(a,k,tau(n))+log(AV_ideal(a,k,tau(n))));  
end;
F = sum(term);
end

